@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -96,7 +96,9 @@ public:
 	void end_call(bool far_end);
 	
 	string get_rel_cause(void) const;
+	string get_rel_cause_internal(void) const;
 	string get_direction(void) const;
+	string get_direction_internal(void) const;
 	bool set_rel_cause(const string &cause);
 	bool set_direction(const string &dir);
 	
