@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ class t_userintf;
 
 // Different types of events
 enum t_event_type {
+	EV_QUIT,		// Generic quit event
 	EV_NETWORK,		// Network event, eg. SIP message from/to network
 	EV_USER,		// User event, eg. SIP message from/to user
 	EV_TIMEOUT,		// Timer expiry
@@ -48,6 +49,7 @@ enum t_event_type {
 	EV_NAT_KEEPALIVE,	// Send a NAT keep alive packet
 	EV_ICMP,		// ICMP error
 	EV_UI,			// User interface event
+	EV_ASYNC_RESPONSE,	// Response on an asynchronous question
 };
 
 ///////////////////////////////////////////////////////////////
@@ -59,6 +61,15 @@ public:
 	virtual t_event_type get_type(void) const = 0;
 };
 
+///////////////////////////////////////////////////////////////
+// Generic quit event
+///////////////////////////////////////////////////////////////
+class t_event_quit : public t_event {
+public:
+	virtual ~t_event_quit();
+	virtual t_event_type get_type(void) const;
+};
+
 ///////////////////////////////////////////////////////////////
 // Network events
 ///////////////////////////////////////////////////////////////
@@ -297,7 +308,8 @@ enum t_ui_event_type {
 	TYPE_UI_CB_LINE_STATE_CHANGED,
 	TYPE_UI_CB_LINE_ENCRYPTED,
 	TYPE_UI_CB_SHOW_ZRTP_SAS,
-	TYPE_UI_CB_ZRTP_CONFIRM_GO_CLEAR
+	TYPE_UI_CB_ZRTP_CONFIRM_GO_CLEAR,
+	TYPE_UI_CB_QUIT
 };
 
 class t_event_ui : public t_event {
@@ -324,6 +336,28 @@ public:
 	void exec(t_userintf *user_intf);
 };
 
+///////////////////////////////////////////////////////////////
+// Asynchronous response event
+///////////////////////////////////////////////////////////////
+class t_event_async_response : public t_event {
+public:
+	enum t_response_type {
+		RESP_REFER_PERMISSION
+	};
+	
+private:
+	t_response_type		response_type;
+	bool			bool_response;
+	
+public:
+	t_event_async_response(t_response_type type);
+	t_event_type get_type(void) const;
+	void set_bool_response(bool b);
+	t_response_type get_response_type(void) const;
+	bool get_bool_response(void) const;
+};
+
+
 ///////////////////////////////////////////////////////////////
 // Event queue
 ///////////////////////////////////////////////////////////////
@@ -348,6 +382,9 @@ public:
 
 	// Push an event into the queue
 	void push(t_event *e);
+	
+	// Push a quit event into the queue
+	void push_quit(void);
 
 	// Create a network event and push it into the queue
 	void push_network(t_sip_message *m, unsigned long ipaddr,
@@ -399,6 +436,9 @@ public:
 	
 	// Create ICMP event
 	void push_icmp(const t_icmp_msg &m);
+	
+	// Create a REFER pemission response event
+	void push_refer_permission_response(bool permission);
 
 	// Pop an event from the queue. If the queue is empty
 	// then the thread will be blocked until an event arrives.
