@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include "hdr_accept.h"
 #include "definitions.h"
 
-t_hdr_accept::t_hdr_accept() : t_header() {};
+t_hdr_accept::t_hdr_accept() : t_header("Accept") {};
 
 void t_hdr_accept::add_media(const t_media &media) {
 	populated = true;
@@ -31,18 +31,6 @@ void t_hdr_accept::set_empty(void) {
 	media_list.clear();
 }
 
-string t_hdr_accept::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	s += "Accept: ";
-	s += encode_value();
-	s += CRLF;
-
-	return s;
-}
-
 string t_hdr_accept::encode_value(void) const {
 	string s;
 	
@@ -51,18 +39,9 @@ string t_hdr_accept::encode_value(void) const {
 	for (list<t_media>::const_iterator i = media_list.begin();
 	     i != media_list.end(); i++)
 	{
-		if (i != media_list.begin()) s += ", ";
+		if (i != media_list.begin()) s += ",";
 		s += i->encode();
 	}
 	
 	return s;
 }
-
-string t_hdr_accept::encode_env(void) const {
-	string s;
-	
-	s = "SIP_ACCEPT=";
-	s += encode_value();
-	
-	return s;
-}
