@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,10 +31,8 @@ public:
 	string	call_id;
 
 	t_hdr_call_id();
-	void set_call_id(string id);
-	string encode(void) const;
+	void set_call_id(const string &id);
 	string encode_value(void) const;
-	string encode_env(void) const;
 };
 
 #endif
