@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,25 +20,13 @@
 #include "hdr_content_language.h"
 #include "util.h"
 
-t_hdr_content_language::t_hdr_content_language() : t_header() {};
+t_hdr_content_language::t_hdr_content_language() : t_header("Content-Language") {};
 
 void t_hdr_content_language::add_language(const t_language &language) {
 	populated = true;
 	language_list.push_back(language);
 }
 
-string t_hdr_content_language::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	s += "Content-Language: ";
-	s += encode_value();
-	s += CRLF;
-
-	return s;
-}
-
 string t_hdr_content_language::encode_value(void) const {
 	string s;
 
@@ -53,12 +41,3 @@ string t_hdr_content_language::encode_value(void) const {
 
 	return s;
 }
-
-string t_hdr_content_language::encode_env(void) const {
-	string s;
-	
-	s = "SIP_CONTENT_LANGUAGE=";
-	s += encode_value();
-	
-	return s;
-}
