@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #include "hdr_cseq.h"
 #include "util.h"
 
-t_hdr_cseq::t_hdr_cseq() : t_header() {
+t_hdr_cseq::t_hdr_cseq() : t_header("CSeq") {
 	seqnr = 0;
 	method = INVITE;
 }
@@ -44,18 +44,6 @@ void t_hdr_cseq::set_method(const string &s) {
 	}
 }
 
-string t_hdr_cseq::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	s = "CSeq: ";
-	s += encode_value();
-	s += CRLF;
-	
-	return s;
-}
-
 string t_hdr_cseq::encode_value(void) const {
 	string s;
 
@@ -67,15 +55,6 @@ string t_hdr_cseq::encode_value(void) const {
 	return s;
 }
 
-string t_hdr_cseq::encode_env(void) const {
-	string s;
-	
-	s = "SIP_CSEQ=";
-	s += encode_value();
-	
-	return s;
-}
-
 bool t_hdr_cseq::operator==(const t_hdr_cseq &h) const {
 	if (method != METHOD_UNKNOWN) {
 		return (seqnr == h.seqnr && method == h.method);
