@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "definitions.h"
 #include "util.h"
 
-t_hdr_date::t_hdr_date() : t_header() {}
+t_hdr_date::t_hdr_date() : t_header("Date") {}
 
 void t_hdr_date::set_date_gm(struct tm *tm) {
 	populated = true;
@@ -38,18 +38,6 @@ void t_hdr_date::set_now(void) {
 	date = t.tv_sec;
 }
 
-string t_hdr_date::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	s = "Date: ";
-	s += encode_value();
-	s += CRLF;
-
-	return s;
-}
-
 string t_hdr_date::encode_value(void) const {
 	string s;
 	struct tm tm;
@@ -74,12 +62,3 @@ string t_hdr_date::encode_value(void) const {
 
 	return s;
 }
-
-string t_hdr_date::encode_env(void) const {
-	string s;
-	
-	s = "SIP_DATE=";
-	s += encode_value();
-	
-	return s;
-}
