@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include "hdr_event.h"
 #include "parse_ctrl.h"
 
-t_hdr_event::t_hdr_event() : t_header() {}
+t_hdr_event::t_hdr_event() : t_header("Event", "o") {}
 
 void t_hdr_event::set_event_type(const string &t) {
 	populated = true;
@@ -36,23 +36,6 @@ void t_hdr_event::add_event_param(const t_parameter &p) {
 	event_params.push_back(p);
 }
 
-string t_hdr_event::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	if (t_parser::compact_headers) {
-		s = "o: ";
-	} else {
-		s = "Event: ";
-	}
-
-	s += encode_value();
-	s += CRLF;
-	
-	return s;
-}
-
 string t_hdr_event::encode_value(void) const {
 	string s;
 
@@ -69,12 +52,3 @@ string t_hdr_event::encode_value(void) const {
 
 	return s;
 }
-
-string t_hdr_event::encode_env(void) const {
-	string s;
-	
-	s = "SIP_EVENT=";
-	s += encode_value();
-	
-	return s;
-}
