@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #include "definitions.h"
 #include "parse_ctrl.h"
 
-t_hdr_from::t_hdr_from() : t_header() {}
+t_hdr_from::t_hdr_from() : t_header("From", "f") {}
 
 void t_hdr_from::set_display(const string &d) {
 	populated = true;
@@ -52,23 +52,6 @@ void t_hdr_from::add_param(const t_parameter &p) {
 	params.push_back(p);
 }
 
-string t_hdr_from::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	if (t_parser::compact_headers) {
-		s = "f: ";
-	} else {
-		s = "From: ";
-	}
-
-	s += encode_value();
-	s += CRLF;
-
-	return s;
-}
-
 string t_hdr_from::encode_value(void) const {
 	string s;
 
@@ -95,15 +78,6 @@ string t_hdr_from::encode_value(void) const {
 	return s;
 }
 
-string t_hdr_from::encode_env(void) const {
-	string s;
-	
-	s = "SIP_FROM=";
-	s += encode_value();
-	
-	return s;
-}
-
 string t_hdr_from::get_display_presentation(void) const {
 	if (display_override.empty()) {
 		return display;
