@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,36 +19,15 @@
 #include "definitions.h"
 #include "hdr_organization.h"
 
-t_hdr_organization::t_hdr_organization() : t_header() {};
+t_hdr_organization::t_hdr_organization() : t_header("Organization") {};
 
 void t_hdr_organization::set_name(const string &n) {
 	populated = true;
 	name = n;
 }
 
-string t_hdr_organization::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	s = "Organization: ";
-	s += encode_value();
-	s += CRLF;
-	
-	return s;
-}
-
 string t_hdr_organization::encode_value(void) const {
 	if (!populated) return "";
 
 	return name;
 }
-
-string t_hdr_organization::encode_env(void) const {
-	string s;
-	
-	s = "SIP_ORGANIZATION=";
-	s += encode_value();
-	
-	return s;
-}
