@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,35 +19,15 @@
 #include "hdr_proxy_authenticate.h"
 #include "definitions.h"
 
-t_hdr_proxy_authenticate::t_hdr_proxy_authenticate() : t_header() {}
+t_hdr_proxy_authenticate::t_hdr_proxy_authenticate() : t_header("Proxy-Authenticate") {}
 
 void t_hdr_proxy_authenticate::set_challenge(const t_challenge &c) {
 	populated = true;
 	challenge = c;
 }
 
-string t_hdr_proxy_authenticate::encode(void) const {
-	string s;
-	if (!populated) return s;
-
-	s += "Proxy-Authenticate: ";
-	s += encode_value();
-	s += CRLF;
-
-	return s;
-}
-
 string t_hdr_proxy_authenticate::encode_value(void) const {
 	if (!populated) return "";
 
 	return challenge.encode();
 }
-
-string t_hdr_proxy_authenticate::encode_env(void) const {
-	string s;
-	
-	s = "SIP_PROXY_AUTHENTICATE=";
-	s += encode_value();
-	
-	return s;
-}
