@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #include "hdr_proxy_authorization.h"
 #include "definitions.h"
 
-t_hdr_proxy_authorization::t_hdr_proxy_authorization() : t_header() {}
+t_hdr_proxy_authorization::t_hdr_proxy_authorization() : t_header("Proxy-Authorization") {}
 
 void t_hdr_proxy_authorization::add_credentials(const t_credentials &c) {
 	populated = true;
@@ -36,7 +36,8 @@ string t_hdr_proxy_authorization::encode(void) const {
 	for (list<t_credentials>::const_iterator i = credentials_list.begin();
 	     i != credentials_list.end(); i++)
 	{
-		s += "Proxy-Authorization: ";
+		s += header_name;
+		s += ": ";
 		s += i->encode();
 		s += CRLF;
 	}
@@ -59,15 +60,6 @@ string t_hdr_proxy_authorization::encode_value(void) const {
 	return s;
 }
 
-string t_hdr_proxy_authorization::encode_env(void) const {
-	string s;
-	
-	s = "SIP_PROXY_AUTHORIZATION=";
-	s += encode_value();
-	
-	return s;
-}
-
 bool t_hdr_proxy_authorization::contains(const string &realm,
 	const t_url &uri) const
 {
