@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,25 +19,13 @@
 #include "definitions.h"
 #include "hdr_proxy_require.h"
 
-t_hdr_proxy_require::t_hdr_proxy_require() : t_header() {};
+t_hdr_proxy_require::t_hdr_proxy_require() : t_header("Proxy-Require") {};
 
 void t_hdr_proxy_require::add_feature(const string &f) {
 	populated = true;
 	features.push_back(f);
 }
 
-string t_hdr_proxy_require::encode(void) const {
-	string s;
-
-	if (!populated) return s;
-
-	s = "Proxy-Require: ";
-	s += encode_value();
-	s += CRLF;
-	
-	return s;
-}
-
 string t_hdr_proxy_require::encode_value(void) const {
 	string s;
 
@@ -52,12 +40,3 @@ string t_hdr_proxy_require::encode_value(void) const {
 
 	return s;
 }
-
-string t_hdr_proxy_require::encode_env(void) const {
-	string s;
-	
-	s = "SIP_PROXY_REQUIRE=";
-	s += encode_value();
-	
-	return s;
-}
