@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,17 +17,51 @@
 */
 
 #include "header.h"
+#include "parse_ctrl.h"
 #include "protocol.h"
 #include "util.h"
 
-t_header::t_header() {
-	populated = false;
+t_header::t_header() :
+	populated(false)
+{}
+
+t_header::t_header(const string &_header_name, const string &_compact_name) :
+	populated(false),
+	header_name(_header_name),
+	compact_name(_compact_name)
+{}
+
+string t_header::encode(void) const {
+	string s;
+
+	if (!populated) return s;
+
+	s = (t_parser::compact_headers && !compact_name.empty() ? 
+			compact_name : header_name);
+	s += ": ";
+	s += encode_value();
+	s += CRLF;
+	
+	return s;
+}
+
+string t_header::encode_env(void) const {
+	string s("SIP_");
+	s += toupper(replace_char(header_name, '-', '_'));
+	s += '=';
+	s += encode_value();
+	
+	return s;
 }
 
 bool t_header::is_populated() const {
 	return populated;
 }
 
+string t_header::get_name(void) const {
+	return header_name;
+}
+
 string t_header::get_value(void) const {
 	string s;
 	int i;
