@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,9 +18,11 @@
 
 #include <assert.h>
 #include <iostream>
+#include <ctime>
 #include "events.h"
 #include "log.h"
 #include "sender.h"
+#include "translator.h"
 #include "userintf.h"
 #include "util.h"
 #include "sockets/socket.h"
@@ -29,6 +31,8 @@
 #include "audits/memman.h"
 #include "stun/stun.h"
 
+#define MAX_TRANSMIT_RETRIES	3
+
 extern t_socket_udp *sip_socket;
 extern t_event_queue *evq_sender_udp;
 extern t_event_queue *evq_trans_mgr;
@@ -63,6 +67,8 @@ static bool handle_socket_err(int err, unsigned long dst_addr, unsigned short ds
 		log_msg += ":";
 		log_msg += int2str(icmp.port);
 		log_msg += "\nSocket error: ";
+		log_msg += int2str(err);
+		log_msg += " ";
 		log_msg += strerror(err);
 		log_file->write_report(log_msg, "::hanlde_socket_err", LOG_NORMAL);
 	
@@ -74,7 +80,7 @@ static bool handle_socket_err(int err, unsigned long dst_addr, unsigned short ds
 		
 		// If the ICMP error comes from the same destination as the
 		// destination of the packet that failed to be sent, then the
-		// packet should be discarded as it can most likely not b
+		// packet should be discarded as it can most likely not be
 		// delivered and would cause an infinite loop of ICMP errors
 		// otherwise.
 		if (icmp.ipaddr == dst_addr && icmp.port == dst_port) {
@@ -85,18 +91,27 @@ static bool handle_socket_err(int err, unsigned long dst_addr, unsigned short ds
 		// Sometimes the error is already present on the socket, but the ICMP
 		// message is not yet queued.
 		log_msg = "Failed to send to SIP UDP socket.\n";
+		log_msg += "Error code: ";
+		log_msg += int2str(err);
+		log_msg += "\n";
 		log_msg += strerror(err);
 		log_file->write_report(log_msg, "::handle_socket_err");
 		
 		num_non_icmp_errors++;
+		
+		/*
+		 * non-ICMP errors occur when a destination on the same
+		 * subnet cannot be reached. So this code seems to be
+		 * harmful.
 		if (num_non_icmp_errors > 100) {
 			log_msg = "Excessive number of socket errors.";
 			log_file->write_report(log_msg, "::handle_socket_err", 
 				LOG_NORMAL, LOG_CRITICAL);
-			return false;
+			log_msg = TRANSLATE("Excessive number of socket errors.");
 			ui->cb_show_msg(log_msg, MSG_CRITICAL);
 			exit(1);
 		}
+		*/
 	}
 	
 	return true;
@@ -121,7 +136,8 @@ static void send_sip_udp(t_event *event) {
 	log_file->write_footer();
 		
 	bool msg_sent = false;
-	while (!msg_sent) {
+	int transmit_count = 0;
+	while (!msg_sent && transmit_count++ <= MAX_TRANSMIT_RETRIES) {
 		try {
 			sip_socket->sendto(e->dst_addr, e->dst_port, m.c_str(), m.size());
 			num_non_icmp_errors = 0;
@@ -130,6 +146,14 @@ static void send_sip_udp(t_event *event) {
 			if (!handle_socket_err(err, e->dst_addr, e->dst_port)) {
 				// Discard packet.
 				msg_sent = true;
+			} else {
+				if (transmit_count <= MAX_TRANSMIT_RETRIES) {
+					// Sleep 100 ms
+					struct timespec sleeptimer;
+					sleeptimer.tv_sec = 0;
+					sleeptimer.tv_nsec = 100000000;
+					nanosleep(&sleeptimer, NULL);
+				}
 			}
 		}
 	}
@@ -159,7 +183,8 @@ static void send_stun(t_event *event) {
 		STUN_MAX_MESSAGE_SIZE, stun_pass, false);
 
 	bool msg_sent = false;
-	while (!msg_sent) {	
+	int transmit_count = 0;
+	while (!msg_sent && transmit_count++ <= MAX_TRANSMIT_RETRIES) {	
 		try {
 			sip_socket->sendto(e->dst_addr, e->dst_port, m, msg_size);
 			num_non_icmp_errors = 0;
@@ -168,6 +193,14 @@ static void send_stun(t_event *event) {
 			if (!handle_socket_err(err, e->dst_addr, e->dst_port)) {
 				// Discard packet.
 				msg_sent = true;
+			} else {
+				if (transmit_count <= MAX_TRANSMIT_RETRIES) {
+					// Sleep 100 ms
+					struct timespec sleeptimer;
+					sleeptimer.tv_sec = 0;
+					sleeptimer.tv_nsec = 100000000;
+					nanosleep(&sleeptimer, NULL);
+				}
 			}
 		}
 	}
@@ -184,7 +217,8 @@ static void send_nat_keepalive(t_event *event) {
 	char m[2] = { '\r', '\n' };
 	
 	bool msg_sent = false;
-	while (!msg_sent) {
+	int transmit_count = 0;
+	while (!msg_sent && transmit_count++ <= MAX_TRANSMIT_RETRIES) {
 		try {
 			sip_socket->sendto(e->dst_addr, e->dst_port, m, 2);
 			num_non_icmp_errors = 0;
@@ -193,6 +227,14 @@ static void send_nat_keepalive(t_event *event) {
 			if (!handle_socket_err(err, e->dst_addr, e->dst_port)) {
 				// Discard packet.
 				msg_sent = true;
+			} else {
+				if (transmit_count <= MAX_TRANSMIT_RETRIES) {
+					// Sleep 100 ms
+					struct timespec sleeptimer;
+					sleeptimer.tv_sec = 0;
+					sleeptimer.tv_nsec = 100000000;
+					nanosleep(&sleeptimer, NULL);
+				}
 			}
 		}
 	}
@@ -200,11 +242,9 @@ static void send_nat_keepalive(t_event *event) {
 
 void *sender_udp(void *arg) {
 	t_event 	*event;
-	t_event_network	*e;
 
-	//t_socket_udp sock;
-
-	while (true) {
+	bool quit = false;
+	while (!quit) {
 		event = evq_sender_udp->pop();
 		
 		switch(event->get_type()) {
@@ -217,11 +257,13 @@ void *sender_udp(void *arg) {
 		case EV_NAT_KEEPALIVE:
 			send_nat_keepalive(event);
 			break;
+		case EV_QUIT:
+			quit = true;
+			break;
 		default:
 			assert(false);
 		}
 
-
 		MEMMAN_DELETE(event);
 		delete event;
 	}
