@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2006  Michel de Boer <michelboer@xs4all.nl>
+    Copyright (C) 2005-2007  Michel de Boer <michel@twinklephone.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -275,7 +275,7 @@ bool t_service::read_config(string &error_msg) {
 		// Skip comment lines
 		if (line[0] == '#') continue;
 
-		list<string> l = split_on_first(line, '=');
+		vector<string> l = split_on_first(line, '=');
 		if (l.size() != 2) {
 			error_msg = "Syntax error in file ";
 			error_msg += f;
@@ -287,8 +287,8 @@ bool t_service::read_config(string &error_msg) {
 			return false;
 		}
 
-		string parameter = trim(l.front());
-		string value = trim(l.back());
+		string parameter = trim(l[0]);
+		string value = trim(l[1]);
 		
 		if (parameter == FLD_CF_ALWAYS) {
 			ui->expand_destination(user_config, value, display_url);
